#!/usr/bin/python
# -*- coding: UTF-8 -*-
# FlameBot, Python IRC bot modular and extensible
#           Main module
#
# Copyright (c) 2005 Diego Pettenò
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# - Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the following
#   disclaimer in the documentation and/or other materials provided
#   with the distribution.
# - The name of the author may not be used to endorse or promote
#   products derived from this software without specific prior
#   written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import ircbot
import irclib
import string

from string import Template
from time import time

# Local imports
from flamebot_utils import *
from xml.dom import minidom

# This can be used to separe the nick from the command
valid_separators = string.whitespace + string.punctuation

class FlameBot(ircbot.SingleServerIRCBot):
	_modules = { }
	_modcommands = { }
	_masters = { }
	_channels = []
	
	def __init__(self):
		prefs = minidom.parse(open("settings.xml")).getElementsByTagName("prefs")[0]
		for chan in prefs.getElementsByTagName("channel"):
			channel = str(getText(chan.childNodes))
			if channel not in self._channels: self._channels = self._channels + [channel]
		
		self._masterpassword = getTextForChild(prefs, "masterpassword", "FlameBot")
		
		server_elem = prefs.getElementsByTagName("server")[0]
		nickname = server_elem.getAttribute("nickname")
		server = server_elem.getAttribute("hostname")
		port = server_elem.getAttribute("port")
		if port == "": port = 6667

		for mod in prefs.getElementsByTagName("module"):
			module = mod.getAttribute("name")
			if module != "":
				self._modules[module] = __import__(module)
				self._modules[module].register_flamebot(self)

		ircbot.SingleServerIRCBot.__init__(self, [(server, port)], nickname, nickname)
	
	def get_version(self):
		return "FlameBot 0.0 by Diego Petteno` <flameeyes@gentoo.org>"

	def on_nicknameinuse(self, c, e):
		c.nick(c.get_nickname() + "_")

	def on_welcome(self, c, e):
		for channel in self._channels:
			c.join(channel)

	def on_privmsg(self, c, e):
		self.do_command(e, e.arguments()[0])

	def on_pubmsg(self, c, e):
		nicklen = len(self.connection.get_nickname())
		
		msg = e.arguments()[0]
		if len(msg) < nicklen: return
		
		maybe_nick = msg[0:nicklen] # Get the first part which *can* be the nick
		command = msg[nicklen+1:].strip()
		
		sep = msg[nicklen]
		
		if maybe_nick == self.connection.get_nickname() and valid_separators.find(sep) != -1:
			self.do_command(e, command, e.target())
		
		return
	
	def addcommand(self, command, cmdcallback):
		self._modcommands[command] = cmdcallback
	
	def removecommand(self, command):
		del self._modcommands[command]

	def do_command(self, e, msg, channel = None):
		nick = irclib.nm_to_n(e.source())
		
		try:
			msg = msg.strip()
			arg = ""
			# Get the command, if there aren't space in command, just take it all
			# else get the first part
			space = msg.find(" ")
			if space == -1:
				cmd = msg
			else:
				cmd = msg[0:space].strip()
				arg = msg[space:].strip()
			
			if cmd == "close":
				if self.check_master(e.source()):
					self.die("I was asked to close myself")
			elif cmd == "join":
				if self.check_master(e.source()):
					self.connection.join(arg)
			elif cmd == "modreload":
				if self.check_master(e.source()) and self._modules.has_key(arg):
					self._modules[arg].unregister(self)
					reload(self._modules[arg])
					self._modules[arg].register_flamebot(self)
			elif cmd == "master":
				if channel != None:
					self.connection.notice(nick, "You fool! Don't tell me that in public.")
					return
				
				self.validate_master(e.source(), arg)
			elif cmd in self._modcommands.keys():
				self._modcommands[cmd](self.connection, nick, channel, arg)
			else:
				self.connection.notice(nick, "Command not understood: " + cmd)
		except Exception, e:
			self.connection.notice(nick, "Error handling request")
			print e
			# raise
	
	def check_master(self, netmask):
		nick = irclib.nm_to_n(netmask)
		if netmask in self._masters.keys():
			if ( time() >= self._masters[netmask] ):
				del self._masters[netmask]
			else:
				return True
		self.connection.notice(nick, "You're not my master.")
		return False

	def validate_master(self, netmask, password):
		nick = irclib.nm_to_n(netmask)
		if password == self._masterpassword:
			self._masters[netmask] = time() + (30*60)
			self.connection.notice(nick, "You've been accepted as a master for 30 minutes.")
		else:
			self.connection.notice(nick, "The password you gave me is not right.")

if __name__ == "__main__":
	FlameBot().start()
